// io_cse.cpp
//
// this module reads .cse files (generated with Win-Stitch: 
// http://www.win-stitch.com/)
//
// current version does not recognize formats other than 2.2, and 
// has no support (and may behave unpredictably) on files with 
// back/quarter stitches
//


//#define _MSP_DEBUG_

#include "vpat_io.h"
#include "io_cse.h"

#ifdef MSDOS
#include <io.h>
#include <string.h>
#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#define O_BINARY 0

#endif

#include <stdio.h>
#include <fcntl.h>

#include <malloc.h>

//
// Is this a WinStitch file?
//
// true if yes, false if no
//
// Check is very simple: if the first bytes of the header are Win-stitch,
// it's a winstitch file.
//
int IsWinStitch(char *chunk, int size)
{
    int i;

    if (0==strncmp(chunk, "Win-stitch", 10))
      {

#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found Win-stitch!\n");
#endif

        return true;
      }
    else
      {
        return false;
      }
}

// Some assumptions:
//     pat is checked to see if it's NULL
//     However, the internals are assumed to be NULL before calling, so 
//     the function calling this is responsible for freeing the pattern 
//     data before reuse
int ReadWinStitch(vpat_chart *pat, char *name)
  {

    int pat_fd, num_read, num_extras, version, num_qst, num_fr_knots, num_beads,
        i, x, y, s;                                // looping variables
    unsigned int stitch_count;
    unsigned char in_data[255];

    if (!pat) return VPATIO_BADPAT;
    pat->palette = NULL;
    pat->stitches = NULL;
    pat->bstitches = NULL;

    if ((pat_fd = open(name, O_RDONLY|O_BINARY)) < 0)
      {
        return VPATIO_OPEN;
      }

    // no running off the end here! (this makes the strncpy safe)
    if (strlen(name) > 255) name[255]=0x00;
    strncpy(pat->header.fname, name, strlen(name));

    num_read=read(pat_fd, in_data, 26);
    if (num_read < 26)
      {
        return VPATIO_SHORT;
      }

    if (strncmp((char *)(&in_data[0]), "Win-stitch", 10)!=0)
      {
        return VPATIO_UNKNOWN;
      }

    num_read=read(pat_fd, in_data, 62);
    if (num_read < 62)
      {
        return VPATIO_SHORT;
      }

    pat->header.height        = (int)in_data[14] + (int)((int) in_data[15]<<8);
    pat->header.width         = (int)in_data[16] + (int)((int) in_data[17]<<8);
    pat->header.num_bstitches = (int)in_data[ 8] + (int)((int) in_data[ 9]<<8);
    pat->header.num_colors    = (int)in_data[ 4] + (int)((int) in_data[ 5]<<8);

    num_qst                   = (int)in_data[ 6] + (int)((int) in_data[ 7]<<8);
    num_fr_knots              = (int)in_data[10] + (int)((int) in_data[11]<<8);
    num_beads                 = (int)in_data[12] + (int)((int) in_data[13]<<8);

#ifdef _MSP_DEBUG_
    fprintf(stderr, "colors: %d, qst: %d, bst: %d, frknots: %d, beads: %d\n", 
                    pat->header.num_colors,
                    num_qst,
                    pat->header.num_bstitches,
                    num_fr_knots,
                    num_beads);
#endif

    pat->header.num_stitches = (unsigned int)pat->header.width * 
                               (unsigned int)pat->header.height;

    pat->stitches = (vpat_stitch *)malloc(sizeof(vpat_stitch) *
                                          pat->header.num_stitches);
    if (!pat->stitches)
      {
        return VPATIO_MALLOC;
      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "reading stitches\n");
#endif

    for (i = 0; i < pat->header.num_stitches; i++)
      {
        num_read=read(pat_fd, in_data, 2);
        if (num_read < 2)
          {
            return VPATIO_SHORT;
          }
        if (in_data[0]==0)
          {
            pat->stitches[i].color = -1;
          }
        else
          {
            pat->stitches[i].color = in_data[0] - 1;
          }
        pat->stitches[i].type = 0;
        pat->stitches[i].second = 0;
      }

#ifdef _MSP_DEBUG_
        fprintf(stderr, "reading specials\n");
#endif

        pat->bstitches = (vpat_bstitch *)malloc((pat->header.num_bstitches +
                                                num_fr_knots +
                                                num_beads) *
                                                (sizeof(vpat_bstitch)));

        if (!pat->bstitches)
          {
            return VPATIO_MALLOC;
          }

#ifdef _MSP_DEBUG_
  fprintf(stderr, "ignoring qstitches\n");
#endif

// FIXME: we are ignoring the quarter-stitch info for now
// it appears to be some sort of (x, y, c) where c is either stitch 
// type (possibly) or color (possible): note also that there is a 
// seemingly apropos value of 0x08 in certain stitch locations -- these 
// may correspond!
        for (i=0; i < num_qst; i++)
          {
            num_read = read(pat_fd, in_data, 8);
 
            if (num_read < 8)
              return VPATIO_SHORT;

          }

        for (i=0; i < pat->header.num_bstitches; i++)
          {
            num_read = read(pat_fd, in_data, 14);
    
            if (num_read < 14)
              return VPATIO_SHORT;

            pat->bstitches[i].color=(int)in_data[0] +
                                   (int)((int) in_data[1] << 8);
            pat->bstitches[i].x1 = ((int)in_data[2] +
                                    (int)((int) in_data[3] << 8))*2;
            pat->bstitches[i].y1 = ((int)in_data[4] +
                                    (int)((int) in_data[5] << 8))*2;
            pat->bstitches[i].x2 = ((int)in_data[6] +
                                    (int)((int) in_data[7] << 8))*2;
            pat->bstitches[i].y2 = ((int)in_data[8] +
                                    (int)((int) in_data[9] << 8))*2;
            pat->bstitches[i].type = 0;   // back stitch

          }

// Gross hack for french knots and beads

    if (num_fr_knots > 0)
      {

#ifdef _MIKE_DEBUG
    fprintf(stderr, "reading knots(?)\n");
#endif
        for (i = 0; i < num_fr_knots; i++)
          {
            num_read = read(pat_fd, in_data, 14);
            if (num_read < 14)
              {
                return VPATIO_SHORT;
              }
            pat->bstitches[i+pat->header.num_bstitches].type = 1;
            pat->bstitches[i+pat->header.num_bstitches].x1 = 2 *
              (int)in_data[2] + (int)((int)in_data[3]<<8);
            pat->bstitches[i+pat->header.num_bstitches].y1 = 2 *
              (int)in_data[4] + (int)((int)in_data[5]<<8);
            pat->bstitches[i+pat->header.num_bstitches].color = 
              (int)in_data[0] + (int)((int)in_data[1]<<1);

          }

        pat->header.num_bstitches += num_fr_knots;

      }

    if (num_beads > 0)
      {
        for (i = 0; i < num_beads; i++)
          {
            num_read = read(pat_fd, in_data, 28);
            if (num_read < 28)
              {
                return VPATIO_SHORT;
              }
            pat->bstitches[i+pat->header.num_bstitches].type = 1;
            pat->bstitches[i+pat->header.num_bstitches].x1 = 2 *
              (int)in_data[2] + (int)((int)in_data[3]<<8);
            pat->bstitches[i+pat->header.num_bstitches].y1 = 2 *
              (int)in_data[4] + (int)((int)in_data[5]<<8);
            pat->bstitches[i+pat->header.num_bstitches].color = 
              (int)in_data[0] + (int)((int)in_data[1]<<1);

          }

        pat->header.num_bstitches += num_beads;

      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "specials read, allocating palette\n");
#endif

    pat->palette = (vpat_color_entry *)malloc(sizeof(vpat_color_entry) *
                                              pat->header.num_colors);
    if (!pat->palette)
      {
        return VPATIO_MALLOC;
      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "palette allocated, reading %d colors\n",
            pat->header.num_colors);
#endif

    for (i=0; i < pat->header.num_colors; i++)
      {
        num_read = read(pat_fd, in_data, 42);
        if (num_read < 2)
          {
            return VPATIO_SHORT;
          }
        pat->palette[i].used_flag = 0;
        pat->palette[i].red = in_data[4];
        pat->palette[i].green = in_data[5];
        pat->palette[i].blue = in_data[6];
//      pat->palette[i].symbol = in_data[37];
        pat->palette[i].symbol = i*3 + 32;

#ifdef _MSP_DEBUG_
//        fprintf(stderr, "[%d,%d,%d]\n",
//                in_data[4], in_data[5], in_data[6]);
#endif

        _colors[i].ResetColor(in_data[4],in_data[5],in_data[6]);
//FIXME: grab "number" from string, and store
        strncpy(pat->palette[i].name, (char *)(&in_data[8]), 26);
        pat->palette[i].DMC_num = 0;
      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "Colors read, ending\n");
#endif

    close(pat_fd);

    return VPATIO_OK;

  }

