// io_pmak.cpp
//
// This module reads PatternMaker 1.0 files.  More information on 
// PatternMaker is available from HobbyWare, the company that puts it out,
// available at http://www.hobbyware.com/
//
// Support for version 3.0 files is due in the next version of vpatview,
// although the beginnings have been implemented (header, palette).
//
// There is no support for 2.0 files (I believe), since I have seen no 
// evidence of them "in the wild" and thus have nothing to work with.
//
// (Version 1.0 was distributed in several ways, including with certain 
//  printers.)
//

//#define _MSP_DEBUG_

#include <stdlib.h>

#include "vpat_io.h"
#include "io_pmak.h"

#ifdef MSDOS
#include <io.h>
#include <string.h>
#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#define O_BINARY 0

#endif

#include <stdio.h>
#include <fcntl.h>

#include <malloc.h>

#ifdef _MSP_DEBUG_
#include <v/vapp.h>
#include <v/vnotice.h>
#endif

//
// Is this a PMaker file?
//
// true if yes, false if no
//
int IsPMaker(char *chunk, int size)
  {

// FIXME: Add support for PatternMaker 2.0 here
    if ( ((chunk[0]==8)&&(chunk[1]==3)) ||  // it's a PatternMaker 1.0 file
         ((chunk[0]==7)&&(chunk[1]==4)) )   // it's a PatternMaker 3.0 file
      {
        return true;
      }
    else
      {
        return false;
      }

  }


int ReadPMaker(vpat_chart *pat, char *name)
  {

    int pat_fd, num_read, num_stitches, num_specials, version, 
        i, ul, ur, ll, lr;
    int *specials;
    unsigned char in_data[256];                 // buffer for read() calls

    if (!pat) { return VPATIO_BADPAT; }
    pat->palette = NULL;
    pat->stitches = NULL;
    pat->bstitches = NULL;

    if ((pat_fd = open(name, O_RDONLY|O_BINARY)) < 0)
      {
        return VPATIO_OPEN;
      }

    // no running off the end here! (this makes the strncpy safe)
    if (strlen(name) > 255) name[255]=0x00;
    strncpy(pat->header.fname, name, strlen(name));

    if ((num_read = read(pat_fd, in_data, 22)) < 22)
      {
        close(pat_fd);
        return VPATIO_SHORT;
      }

    if ( (in_data[0]==8)&&(in_data[1]==3) )
      {
        version = 1;  // PatternMaker 1.0 file
      }
// FIXME: add support for 2.0 here!
    else if ( (in_data[0]==7)&&(in_data[1]==4) )
      {
#ifdef _MSP_DEBUG_
vNoticeDialog note(theApp); // notice instance
(void)note.Notice("PMAK3");
#endif
        version = 3;  // PatternMaker 3.0 file
      }
    else
      {
        close(pat_fd);
        return VPATIO_UNKNOWN;
      }

	// read in header information
    switch (version)
      {
        case 1:
          pat->header.width  = (int)in_data[6] + (int)((int)in_data[7] << 8);
          pat->header.height = (int)in_data[8] + (int)((int)in_data[9] << 8);
          num_specials       = (int)in_data[10]+ (int)((int)in_data[11]<< 8);
          pat->header.num_colors = (int)in_data[20] + 
                               (int)((int)in_data[21] << 8);
          pat->header.num_bstitches = (int)in_data[12] + 
                               (int)((int)in_data[13] << 8);
          break;

        // FIXME: add support for version 2

        case 3:
          // these are actually the same headers!
          pat->header.width  = (int)in_data[ 6] + (int)((int)in_data[ 7] << 8);
          pat->header.height = (int)in_data[ 8] + (int)((int)in_data[ 9] << 8);
          num_specials       = (int)in_data[10] + (int)((int)in_data[11] << 8);
          pat->header.num_colors  = (int)in_data[20] + 
                               (int)((int)in_data[21] << 8);
          pat->header.num_bstitches  = (int)in_data[12] + 
                               (int)((int)in_data[13] << 8);
          break;

        default:
          fprintf(stderr, "Unknown PatternMaker version %d\n", version);
		  return VPATIO_UNKNOWN;
          break;

      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "w: %d, h: %d, specials: %d, nc: %d, bst: %d\n", 
            pat->header.width, 
            pat->header.height,
            num_specials,
            pat->header.num_colors,
            pat->header.num_bstitches);
/* 
{
	char tmpbuf[256];
	vNoticeDialog note(theApp);

    sprintf(tmpbuf, "w: %d, h: %d, specials: %d, nc: %d, bst: %d\0", 
            pat->header.width, 
            pat->header.height,
            num_specials,
            pat->header.num_colors,
            pat->header.num_bstitches);
	(void)note.Notice(tmpbuf);
}
// */
#endif

    specials = NULL;
    if (num_specials > 0)
      {
        specials = (int *)malloc(sizeof(int)*num_specials);
        if (!specials)
          {
            close(pat_fd);
            return VPATIO_MALLOC;
          }
      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "specials...");
#endif

    if (!(pat->palette = (vpat_color_entry *)malloc(sizeof(vpat_color_entry) *
                         pat->header.num_colors)))
      {
        close(pat_fd);
        return VPATIO_MALLOC;
      }

#ifdef _MSP_DEBUG_
    fprintf(stderr, "palette...");
#endif

	// read in palette
    switch (version)
      {
        case 1:
          for (i=0; i<pat->header.num_colors; i++)
            {
              if ((num_read=read(pat_fd, in_data, 28))<28)
                {
                  close(pat_fd);
                  return VPATIO_SHORT;
                }

              pat->palette[i].DMC_num = (int)in_data[1] + (int)((int)in_data[2]<<8);
              pat->palette[i].red   = in_data[24];
              pat->palette[i].green = in_data[25];
              pat->palette[i].blue  = in_data[26];
              pat->palette[i].used_flag = 0;
              strncpy(pat->palette[i].name, (char *)(&in_data[3]), 21);
              _colors[i].ResetColor(in_data[24],in_data[25],in_data[26]);
            }

          num_read=read(pat_fd, in_data, pat->header.num_colors*2);
          num_read=read(pat_fd, in_data, pat->header.num_colors*2);
          for (i=0; i < pat->header.num_colors; i++)
            {
              pat->palette[i].symbol = in_data[i*2];
            }
          break;

        case 3:
          for (i=0; i<pat->header.num_colors; i++)
            {
              if ((num_read=read(pat_fd, in_data, 111)) < 111)
                {
                  close(pat_fd);
                  return VPATIO_SHORT;
                }

              pat->palette[i].DMC_num = atoi((char *)&in_data[1]);

              pat->palette[i].red   = in_data[53];
              pat->palette[i].green = in_data[54];
              pat->palette[i].blue  = in_data[55];
              pat->palette[i].used_flag = 0;
              strncpy(pat->palette[i].name, (char *)(&in_data[12]), 21);
              _colors[i].ResetColor(in_data[53],in_data[54],in_data[55]);

#ifdef _MSP_DEBUG_
              fprintf(stderr, "%s: %s\n", (char *)&in_data[1],
                                          (char *)&in_data[12]);
#endif

            }

          num_read=read(pat_fd, in_data, pat->header.num_colors*2);
//          num_read=read(pat_fd, in_data, pat->header.num_colors*2);
          for (i=0; i < pat->header.num_colors; i++)
            {
               pat->palette[i].symbol = in_data[i*2];
            }

		  // complicated formula to skip ahead -- see pmak3.doc for why
		  int jump_ahead;
		  jump_ahead = (35*pat->header.num_colors) + 3000;

          // jump ahead to the fonts
		  if ((num_read=lseek(pat_fd,jump_ahead,SEEK_CUR)) < jump_ahead )
			{
			  close(pat_fd);
			  return VPATIO_SHORT;
			}

          break;

      } // switch (version)

// skip some chunks


// skip the fonts -- a "smart" implementation would take some guesses and attempt to 
// assign reasonable values for symbol/text fonts?

// note: this skips the fonts (98 bytes in toto, 49 each)
//       same length of storage for Version 1 and 3
    num_read=read(pat_fd, in_data, 49);
	num_read=read(pat_fd, in_data, 49);
#ifdef _MSP_DEBUG_
	{
		vNoticeDialog note(theApp);
		(void)note.Notice((char *)(&in_data[0]));
	}
#endif

// note: this skips whole chunks of data, probably settings stuff, 
//       ending (hopefully) at the background color
	switch (version)
	{
	case 1:

		if ((num_read=lseek(pat_fd,312,SEEK_CUR))<312)
		  {
			close(pat_fd);
			return VPATIO_SHORT;
		  }
		break;

	case 3:

		if ((num_read=lseek(pat_fd,313,SEEK_CUR))<313)
		{
			close(pat_fd);
			return VPATIO_SHORT;
		}
	} // switch

// read in background color
	switch (version)
	{
	case 1:
		if ((num_read=read(pat_fd, in_data, 28))<28)
		  {
			close(pat_fd);
			return VPATIO_SHORT;
		  }

        pat->header.background.DMC_num = (int)in_data[1] + 
                                         (int)((int)in_data[2]<<8);
		pat->header.background.red   = in_data[24];
		pat->header.background.green = in_data[25];
		pat->header.background.blue  = in_data[26];
		strncpy(pat->header.background.name, (char *)(&in_data[3]), 21);
		_bgcolor.ResetColor(in_data[24],in_data[25],in_data[26]);
		break;
		
	case 3:
		if ((num_read=read(pat_fd, in_data, 111))<111)
		  {
			close(pat_fd);
			return VPATIO_SHORT;
		  }

        pat->header.background.DMC_num = atoi((char *)&in_data[1]);
		pat->header.background.red   = in_data[54];
		pat->header.background.green = in_data[55];
		pat->header.background.blue  = in_data[56];
		strncpy(pat->header.background.name, (char *)(&in_data[12]), 21);
		_bgcolor.ResetColor(in_data[54],in_data[55],in_data[56]);

		break;

	} // switch

// skip some more huge chunks, to get to the actual data
	switch (version)
	{
	case 1:
	    if ((num_read=lseek(pat_fd,2738,SEEK_CUR))<2738)
		  {
			close(pat_fd);
	        return VPATIO_SHORT;
		  }
		break;

	case 3:
	    if ((num_read=lseek(pat_fd,5303,SEEK_CUR))<5303)
		  {
			close(pat_fd);
	        return VPATIO_SHORT;
		  }
		break;

	} // switch

    num_stitches = pat->header.width * pat->header.height;
    if (!(pat->stitches = (vpat_stitch *)malloc(sizeof(vpat_stitch) * 
                          num_stitches)))
      {
        close(pat_fd);
        return VPATIO_MALLOC;
      }

    for (i=0; i < num_stitches; i++)
      {
        num_read=read(pat_fd, in_data, 2);
        if ((in_data[1]&0x0F)==0x0F)               // if flagged as unused
          { 
            pat->stitches[i].color=-1;             // mark as "invisible"
          }
        else if ((in_data[1]&0x80)==0x80)
          {
            specials[(int)in_data[0] + (int)((int)(in_data[1]&0x03)<<8)]=i;
          }
        else
          {
            pat->stitches[i].color=in_data[0];
            pat->stitches[i].type=0;
          }
      }

    for (i=0; i<num_specials; i++)
      {

        num_read=read(pat_fd, in_data, 9);

        if (in_data[0]&0x40)
          {

            ul = in_data[0] & 0x04;
            ll = in_data[0] & 0x08;
            ur = in_data[0] & 0x10;
            lr = in_data[0] & 0x20;

            if (ul&&lr)
              {
                pat->stitches[specials[i]].type = 7;
                pat->stitches[specials[i]].color = (int)in_data[4];
                pat->stitches[specials[i]].second =(int)in_data[7];
              }
            else if (ll&&ur)
              {
                pat->stitches[specials[i]].type = 8;
                pat->stitches[specials[i]].color = (int)in_data[6];
                pat->stitches[specials[i]].second =(int)in_data[5];
              }
            else
              {
                if (ul)
                  {
                    pat->stitches[specials[i]].type = 1;
                    pat->stitches[specials[i]].color = (int)in_data[4];
                    pat->stitches[specials[i]].second = 0;
                  }
                else if (ur)
                  {
                    pat->stitches[specials[i]].type = 2;
                    pat->stitches[specials[i]].color = (int)in_data[6];
                    pat->stitches[specials[i]].second = 0;
                  }
                else if (ll)
                  {
                    pat->stitches[specials[i]].type = 3;
                    pat->stitches[specials[i]].color = (int)in_data[5];
                    pat->stitches[specials[i]].second = 0;
                  }
                else if (lr)
                  {
                    pat->stitches[specials[i]].type = 4;
                    pat->stitches[specials[i]].color = (int)in_data[7];
                    pat->stitches[specials[i]].second = 0;
                  }

              }
          }

        else
          {
            fprintf(stderr, "Unknown special...\n");
          }

      }

    pat->bstitches = NULL;

    if (pat->header.num_bstitches > 0)
      {

        if (!(pat->bstitches = (vpat_bstitch *)malloc(sizeof(vpat_bstitch) *
                                                pat->header.num_bstitches +1)))
          {
            close(pat_fd);
            return VPATIO_MALLOC;
          }

        for (i=0; i < pat->header.num_bstitches; i++)
          {
            num_read=read(pat_fd, in_data, 12);

            if (num_read==12)
              {
                pat->bstitches[i].color=(int)in_data[10] +
                                       (int)((int) in_data[11] << 8);
                if (in_data[0]==2)            // Ordinary backstitch
                  {
					// this multiplies the (x,y) by 2 for v1, but not for v3.
					// to support the alternate methods of handling backstitches
					switch (version)
					{
					case 1:
	                    pat->bstitches[i].type = 0;
		                pat->bstitches[i].x1 = ((int)in_data[2] +
			                                    (int)((int) in_data[3] << 8))*2;
				        pat->bstitches[i].y1 = ((int)in_data[4] +
					                            (int)((int) in_data[5] << 8))*2;
						pat->bstitches[i].x2 = ((int)in_data[6] +
							                    (int)((int) in_data[7] << 8))*2;
	                    pat->bstitches[i].y2 = ((int)in_data[8] +
		                                        (int)((int) in_data[9] << 8))*2;
						break;

					case 3:
	                    pat->bstitches[i].type = 0;
		                pat->bstitches[i].x1 = ((int)in_data[2] +
			                                    (int)((int) in_data[3] << 8));
				        pat->bstitches[i].y1 = ((int)in_data[4] +
					                            (int)((int) in_data[5] << 8));
						pat->bstitches[i].x2 = ((int)in_data[6] +
							                    (int)((int) in_data[7] << 8));
	                    pat->bstitches[i].y2 = ((int)in_data[8] +
		                                        (int)((int) in_data[9] << 8));
						break;
					} // switch
                  }
                else if (in_data[0]==1)       // French Knot
                  {
					switch (version)
					{
					case 1:
						pat->bstitches[i].type = 1;
						pat->bstitches[i].x1 = ((int)in_data[2] +
												(int)((int) in_data[3] << 8))*2;
						pat->bstitches[i].y1 = ((int)in_data[4] +
												(int)((int) in_data[5] << 8))*2;
						pat->bstitches[i].x2 = 0;
						pat->bstitches[i].y2 = 0;
						break;

					case 3:
						pat->bstitches[i].type = 1;
						pat->bstitches[i].x1 = ((int)in_data[2] +
												(int)((int) in_data[3] << 8))*2;
						pat->bstitches[i].y1 = ((int)in_data[4] +
												(int)((int) in_data[5] << 8))*2;
						pat->bstitches[i].x2 = 0;
						pat->bstitches[i].y2 = 0;
						break;

					} // switch
                  }
                else 
                  {
                    fprintf(stderr, "Unknown backstitch type %d\n", in_data[0]);
                    pat->bstitches[i].type = -1;  // who knows what it is
                  }
              }
            else return VPATIO_READ;
          }

      }

    close(pat_fd);

    return VPATIO_OK;

  }

