// io_xsw.cpp
//
// loads X-Stitch for Windows files

//#define _MSP_DEBUG_

#include "vpat_io.h"
#include "io_xsw.h"

#ifdef MSDOS
#include <io.h>
#include <string.h>
#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#define O_BINARY 0

#endif

#include <stdio.h>
#include <fcntl.h>

#include <malloc.h>

//
// Is this an XSWin file?
//
// true if yes, false if no
//
// Check is very simple: first four bytes are: 0x58 0x57 0x32 0x30 (XW20)
// There may be other versions, but I haven't seen them yet
//
int IsXSWin(char *chunk, int size)
{

    if ((chunk[0]==0x58) && (chunk[1]==0x57) &&
        (chunk[2]==0x32) && (chunk[3]==0x30))
      {
        return true;
      }
    else
      {
        return false;
      }

}

// Some assumptions:
//     pat is checked to see if it's NULL
//     However, the internals are assumed to be NULL before calling, so 
//     the function calling this is responsible for freeing the pattern 
//     data before reuse
int ReadXSWin(vpat_chart *pat, char *name)
  {

    int pat_fd, num_read, num_extras, version, num_colors, 
        i, x, y, s;                                // looping variables
    unsigned int stitch_count;
    unsigned char in_data[256];

    if (!pat) return VPATIO_BADPAT;
    pat->palette = NULL;
    pat->stitches = NULL;
    pat->bstitches = NULL;

    if ((pat_fd = open(name, O_RDONLY|O_BINARY)) < 0)
      {
        return VPATIO_OPEN;
      }

    // no running off the end here! (this makes the strncpy safe)
    if (strlen(name) > 255) name[255]=0x00;
    strncpy(pat->header.fname, name, strlen(name));

    num_read=read(pat_fd, in_data, 131);
    if (num_read < 131)
      {
        return VPATIO_SHORT;
      }

    pat->header.height        = (int)in_data[7] + 
                                (int)((int) in_data[8] << 8);
    pat->header.width         = (int)in_data[5] + 
                                (int)((int) in_data[6] << 8);

// now, we read the next 120 bytes in; this is the descriptive text which
// accompanies the pattern.

    for (i=0 ; i<256; i++) pat->header.descr[i]=0;
    strncpy((char *)&(pat->header.descr), (char *)&in_data[11], 120);

    pat->header.num_stitches = (unsigned int)(pat->header.width * 
                               pat->header.height);

    pat->stitches = (vpat_stitch *)malloc(sizeof(vpat_stitch) *
                                          pat->header.num_stitches);

    if (!pat->stitches)
      {
        return VPATIO_MALLOC;
      }

    for (x = 0; x < pat->header.width; x++)
      {

        for (y=0; y < pat->header.height; y++)
          {

            num_read = read(pat_fd, in_data, 1);

            if (num_read < 1)
              {
                close(pat_fd);
                return VPATIO_SHORT;
              }

            if (in_data[0] == 0xFF)
              {
                pat->stitches[y*pat->header.width + x].color = -1;
              }
            else
              {
                pat->stitches[y*pat->header.width + x].color = in_data[0];
              }
          }

        for (y=0; y < pat->header.height; y++)
          {
            num_read = read(pat_fd, in_data, 1);

            switch (in_data[0])
              {

                case 0x00:
                  pat->stitches[y*pat->header.width + x].type = 0;
                  break;

                case 0x08:
                  pat->stitches[y*pat->header.width + x].type = 1;
                  break;

                case 0x0A:
                  pat->stitches[y*pat->header.width + x].type = 2;
                  break;

                case 0x0C:
                  pat->stitches[y*pat->header.width + x].type = 3;
                  break;

                case 0x0E:
                  pat->stitches[y*pat->header.width + x].type = 4;
                  break;


                default:
                  break;

              }
          }

      }

    num_read = read(pat_fd, in_data, 2);
    num_colors = int(in_data[0]);

    pat->palette = (vpat_color_entry *)malloc(64*sizeof(vpat_color_entry));
    if (!pat->palette) return VPATIO_MALLOC;

    for (i = 0; i < 64; i++)
      {
        num_read = read(pat_fd, in_data, 3);
        pat->palette[i].red = in_data[0];
        pat->palette[i].green = in_data[1];
        pat->palette[i].blue = in_data[2];
        _colors[i].ResetColor(in_data[0], in_data[1], in_data[2]);
        if (i < num_colors)
          {
            pat->palette[i].used_flag = 0;
          }
        else
          {
            pat->palette[i].used_flag = 0xFF;
          }

// *FIXME* get the actual symbol assignments!
        pat->palette[i].symbol = i + ' ';
        pat->palette[i].DMC_num = 0;
        sprintf(pat->palette[i].name, "XSW names unsupported");

      }

// skip the remainder of the RGB entries
//  lseek(pat_fd, 3*(64 - num_colors), SEEK_CUR);

// This part appears to be a buffer of 36 * 16 (= 576, or 0x240) 0's in all 
// sample files.  As a side note, there appear to be a lot of similar cases 
// of padding, but that may be due to non-complex files.
    lseek(pat_fd, 576, SEEK_CUR);

// 2 byte (LSB) # of backstitches, 3 byte RGB for fabric color
    num_read = read(pat_fd, in_data, 6);

    if (num_read < 6)
      {
        fprintf(stderr, "bsthead fail: %d\n", num_read);
        return VPATIO_SHORT;
      }
    _bgcolor.ResetColor(in_data[2], in_data[3], in_data[4]);

    pat->header.num_bstitches = (int)in_data[0] + (int)((int)in_data[1]<<8);

    if (pat->header.num_bstitches > 0)
      {
        pat->bstitches = (vpat_bstitch *)malloc(sizeof(vpat_bstitch) *
                                                pat->header.num_bstitches);
        if (!pat->bstitches) return VPATIO_MALLOC;

// Let's try this (could be!)
        for (i = 0; i < pat->header.num_bstitches; i++)
          {
            num_read = read(pat_fd, in_data, 2);
            pat->bstitches[i].x1 = ((int)in_data[0] + ((int)in_data[1]<<8));
            pat->bstitches[i].color = -1;
            pat->bstitches[i].type = 0;   // mark it a back stitch
          }

        for (i = 0; i < pat->header.num_bstitches; i++)
          {
            num_read = read(pat_fd, in_data, 2);
            pat->bstitches[i].x2 = ((int)in_data[0] + ((int)in_data[1]<<8));
          }

        for (i = 0; i < pat->header.num_bstitches; i++)
          {
            num_read = read(pat_fd, in_data, 2);
            pat->bstitches[i].y1 = ((int)in_data[0] + ((int)in_data[1]<<8));
          }

        for (i = 0; i < pat->header.num_bstitches; i++)
          {
            num_read = read(pat_fd, in_data, 2);
            pat->bstitches[i].y2 = ((int)in_data[0] + ((int)in_data[1]<<8));
          }
      }

    close(pat_fd);
    return VPATIO_OK;

}
