//=======================================================================
//      vpat_cmd.cpp:   Source for vpatviewCmdWindow class
//=======================================================================

#include <v/vfilesel.h> // for vFileSelect
#include <v/vnotice.h>  // for vNoticeDialog
#include <v/vkeys.h>    // to map keys
#include <v/vicon.h>    // for icons

#include "vpat_cmd.h"   // our header
#include "vpat_io.h"    // for chart I/O operations

#include <malloc.h>     // we need this

#ifdef _MIKE_
#include "io_vst.h"
#endif

#ifdef _SAMPLE_SYMBOLS_
#include <v/vprinter.h>
#include <v/vprintdc.h>
#include "vpat_fnt.h"
#endif

// Set up the icons for the command bar:
#include "buttons/open.vbm"
    static vIcon openI(&open_bits[0],open_height,open_width);
#include "buttons/print.vbm"
    static vIcon printI(&print_bits[0],print_height,print_width);
#include "buttons/page.vbm"
    static vIcon pageI(&page_bits[0],page_height,page_width);
#include "buttons/quit.vbm"
    static vIcon quitI(&quit_bits[0],quit_height,quit_width);


// Start defines for the main window with 100

    enum {
        m_FirstCmd = 100, // Dummy Command
        btnTestTool,	// Tool Bar test
        lblTestStat,	// Status Bar test
        blkLast		// Last item
      };

    enum {
        m_FirstMenuItem = 1000, // Dummy to start enum high enough
        m_PageLayout,
        m_SampleSymbols
    };

    static vMenu FileMenu[] =
      {
        {"&Open...", M_Open, isSens, notChk, noKeyLbl, noKey, noSub},
#ifdef _MIKE_
        {"&Save As...", M_Save, isSens, notChk, noKeyLbl, noKey, noSub},
#endif
        {"Page &Layout", m_PageLayout, isSens, notChk, noKeyLbl, noKey, noSub},
        {"&Print...", M_Print, isSens, notChk, noKeyLbl, noKey, noSub},
        {"-", M_Line, notSens, notChk, noKeyLbl, noKey, noSub},
        {"&About", M_About, isSens, notChk, noKeyLbl, noKey, noSub},
#ifdef _SAMPLE_SYMBOLS_
        {"-", M_Line, notSens, notChk, noKeyLbl, noKey, noSub},
        {"Sample Symbols",m_SampleSymbols,isSens,notChk,noKeyLbl,noKey,noSub},
#endif
        {"-", M_Line, notSens, notChk, noKeyLbl, noKey, noSub},
        {"E&xit", M_Exit, isSens, notChk, noKeyLbl, noKey, noSub},
        {NULL}
      };

    static vMenu StandardMenu[] =
      {
        {"&File", M_File, isSens, notUsed, notUsed, noKey, &FileMenu[0]},
        {NULL}
      };

    static CommandObject ToolBar[] =
      {
        {C_IconButton,M_Open,M_Open,"",&openI,CA_None,isSens,0,0,0},
        {C_IconButton,m_PageLayout,m_PageLayout,"",&pageI,CA_None,isSens,0,0,0},
        {C_IconButton,M_Print,M_Print,"",&printI,CA_None,isSens,0,0,0},
#ifdef _MIKE_
        {C_ColorButton,M_Exit,M_Exit,"Exit",(void *)&vStdColors[vC_Yellow],
                       CA_None,isSens,0,0,0},
#endif
        {C_EndOfList,0,0,0,0,CA_None,0,0,0}
      };

    static vStatus StatBar[] =
      {
        {"V Pattern Viewer", lblTestStat, CA_NoBorder, isSens, 0},
        {0,0,0,0,0}
      };


// Filtering stuff (opening files)
    static char* filter[] =
      {
#ifdef MSDOS
        "*.pat",
        "*.vst", 
        "*.xsw",
	"*.cse",
        "*.pt5",
#else
        "*.cse *.pat *.pt5 *.vst *.xsw",
        "*.CSE *.PAT *.PT5 *.VST *.XSW",
#endif
        0
      };
    int fi = 0;


//===============>>> vpatviewCmdWindow::vpatviewCmdWindow <<<==================
  vpatviewCmdWindow::vpatviewCmdWindow(char* name, int width, int height) :
    vCmdWindow(name, width, height)
  {

    // The Menu Bar
    vpatviewMenu = new vMenuPane(StandardMenu);
    AddPane(vpatviewMenu);

    // The Command Pane
    vpatviewCmdPane = new vCommandPane(ToolBar);
    AddPane(vpatviewCmdPane);

    // The Canvas
    vpatviewCanvas = new vpatviewCanvasPane;
    AddPane(vpatviewCanvas);

    // The Status Bar
    vpatviewStatus = new vStatusPane(StatBar);
    AddPane(vpatviewStatus);

    // Associated dialogs
    aboutDialog = new aboutModalDialog(this);
    pageDialog = new pageModalDialog(this);
    
    // Show Window

    ShowWindow();

  }

//===============>>> vpatviewCmdWindow::~vpatviewCmdWindow <<<=================
  vpatviewCmdWindow::~vpatviewCmdWindow()
  {

    // Now put a delete for each new in the constructor.

    delete vpatviewMenu;
    delete vpatviewCanvas;
    delete vpatviewCmdPane;
    delete vpatviewStatus;

  }

//=====================>>> vpatviewCmdWindow::KeyIn <<<========================
  void vpatviewCmdWindow::KeyIn(vKey keysym, unsigned int shift)
  {
    vCmdWindow::KeyIn(keysym, shift);
  }



//==================>>> vpatviewCmdWindow::WindowCommand <<<====================
  void vpatviewCmdWindow::WindowCommand(ItemVal id, ItemVal val, CmdType cType)
  {
    // Default: route menu and toolbar commands here

    switch (id)
      {
        // Open pattern file
        case M_Open:
          {
            vFileSelect fsel(this);     // file selection dialog box
            char name[100] = "";        // null name
            VCursor old_cursor;

            int oans = fsel.FileSelect("Open Pattern", name, 99,
                                        filter, fi);

            vNoticeDialog fsnote(this); // notice instance
            if (oans && *name)
              {
                int i;
                vpat_chart *pat;

                SetTitle(name);

                pat = (vpat_chart *)malloc(sizeof(vpat_chart));

                if (!pat)
                  {
                    (void)fsnote.Notice("Unable to allocate memory");
                    break;
                  }

                old_cursor = vpatviewCanvas->GetCursor();
                vpatviewCanvas->SetCursor(VC_Wait);

                i = vpat_ReadChart(pat, name);

                switch (i)
                  {
                    case VPATIO_OK:
                      vpatviewCanvas->SetChart(pat);
                      vpatviewCanvas->UpdateOptions(pageDialog);
                      vpatviewCanvas->UpdateScreen();
                      break;

                    default:
                      (void)fsnote.Notice(vpat_IOErrString(i)); 
                      break;
                  }

                vpatviewCanvas->SetCursor(old_cursor);

              }
        
            break;
          }

#ifdef _MIKE_
        // Save as PCStitch file (temporary)
        case M_Save:                  // save file as
          {
            if (_pattern_loaded)
              {
                vFileSelect fsel(this);     // V file select dialog
                char _save_fn[256];
                vNoticeDialog note(this);   // notice instance
                int i;

                _save_fn[0]=0x00;

                if (!fsel.FileSelectSave("Save As vStitch File",
                  _save_fn,99,filter,fi))
                    break;                  // ignore if no selection
                if (_save_fn[0]==0x00) break;

                i = WriteVStitch(vpatviewCanvas->GetChart(), _save_fn);
                if (i != VPATIO_OK)
                  {
                    note.Notice(vpat_IOErrString(i));
                    break;
                  }
              }
            break;
          }
#endif

        // Print the pattern
        case M_Print:
          {
            vpatviewCanvas->Print();

            break;
          }

#ifdef _SAMPLE_SYMBOLS_
        case m_SampleSymbols:
          {
            vPrintDC pdc;
            vPrinter printer;

            if (!printer.Setup("symbols.ps")) return;
            pdc.SetPrinter(printer);        // change to printer that we
                                            // just setup

            if (!pdc.BeginPrinting()) return;       // print to test.ps

            vfnt_symbol_chart(&pdc);

            pdc.EndPrinting();

            break;
 
          }
#endif

        // Print Options dialog
        case m_PageLayout:
          {
            ItemVal val, id;
            id = pageDialog->ShowModalDialog("Print Options",val);
            vpatviewCanvas->UpdateOptions(pageDialog);
// not needed since this only affects printing
//          vpatviewCanvas->UpdateScreen();
            break;
          }

        // About this program
        case M_About:
          {
            ItemVal val, id;
            id = aboutDialog->ShowModalDialog("About vpatview",val);
            break;
          }

        // Leave the program <sniff> they don't love us...
        case M_Exit:
          {
            theApp->Exit();
            break;
          }

        default:                // route unhandled commands up the chain
          {
            vCmdWindow::WindowCommand(id, val, cType);
            break;
          }
      }
  }
