//=============================================================================
//
//      vpat_cnv.cpp:   Source for vpatviewCanvasPane class
//
// This file contains the displaying and printing information for the 
// charts.  Image loading is separate.  In a well-designed application, 
// the chart info would be stored in the AppWinInfo class rather than 
// in the CanvasPane.  This may be corrected in a newer release, if I 
// ever manage to get it working correctly.  Sample code from anyone 
// who actually gets a derived AppWinInfo class working would be 
// greatly appreciated!
//
//=============================================================================

//#define _MSP_DEBUG_

#include "vpat_cnv.h"
#include "vpat_fnt.h"

#ifdef MSDOS
#include <stdio.h>
#endif

#include <malloc.h>

//==============>>> vpatviewCanvasPane::vpatviewCanvasPane <<<=================
  vpatviewCanvasPane::vpatviewCanvasPane()
  {

    _vmem_x = 1; _vmem_y = 1;
    _vmem = new vMemoryDC(_vmem_x, _vmem_y);

    _font = new vFont(vfDefaultSystem, 10, vfNormal, vfNormal, 0);
    _symfont = new vFont(vfDecorative, 10, vfNormal, vfNormal, 0);

    _scale_by = 10;
    _print_chart = 1; // yes, default to printing the chart
    _print_type = 0;  // colored box
    _show_grid = 1;   // default to showing grid

    _print_colors_first = 1;
    _print_colors_last = 0;

    _int_font = 0;

    _bgcolor.ResetColor(255,255,255);

    _pat = NULL;

  }


//=============>>> vpatviewCanvasPane::~vpatviewCanvasPane <<<=================
  vpatviewCanvasPane::~vpatviewCanvasPane()
  {

    delete _font;
    delete _symfont;
    delete _vmem;

    if (_pat) free(_pat);

  }


//=====================>>> vpatviewCanvasPane::HPage <<<=======================
  void vpatviewCanvasPane::HPage(int shown, int top)
  {
    vCanvasPane::HPage(shown,top);
  }


//=====================>>> vpatviewCanvasPane::VPage <<<=======================
  void vpatviewCanvasPane::VPage(int shown, int top)
  {
    vCanvasPane::VPage(shown,top);
  }


//=====================>>> vpatviewCanvasPane::HScroll <<<=====================
  void vpatviewCanvasPane::HScroll(int step)
  {
    vCanvasPane::HScroll(step);
  }


//====================>>> vpatviewCanvasPane::VScroll <<<======================
  void vpatviewCanvasPane::VScroll(int step)
  {
    vCanvasPane::VScroll(step);
  }


//===================>>> vpatviewCanvasPane::MouseDown <<<=====================
  void vpatviewCanvasPane::MouseDown(int X, int Y, int button)
  {
    vCanvasPane::MouseDown(X,Y,button);
  }


//=====================>>> vpatviewCanvasPane::MouseUp <<<=====================
  void vpatviewCanvasPane::MouseUp(int X, int Y, int button)
  {
    vCanvasPane::MouseUp(X,Y,button);
  }


//===================>>> vpatviewCanvasPane::MouseMove <<<=====================
  void vpatviewCanvasPane::MouseMove(int x, int y, int button)
  {
    vCanvasPane::MouseMove(x,y,button);
  }


//======================>>> vpatviewCanvasPane::Redraw <<<=====================
  void vpatviewCanvasPane::Redraw(int x, int y, int w, int h)
  {

    // due to the order of initialization, this is necessary to 
    // "create" _vmem at the correct size:  (V does not initialize the
    // DC until *after* the initialization of the class (of course), 
    // so creating the vMemoryDC to the screen size is done here, 
    // and Redraw is called during the course of initializing the app

    // Note that we could also initialize it to some "reasonable" value, 
    // like 1024x768, but this is less wasteful, and it practically 
    // guarantees that _vmem will be created at some reasonable size

    if (_vmem_x == 1)
      {
        vDC *dc;

        delete _vmem;

        dc = GetDC();

        _vmem_x = dc->GetPhysWidth();      // this returns the screen 
        _vmem_y = dc->GetPhysHeight();     // dimensions

        _vmem = new vMemoryDC(_vmem_x, _vmem_y);
        _vmem->SetBackground(vStdColors[vC_White]);
        _vmem->Clear();                    // otherwise, you get the 
                                           // display background

      }

    // display from _vmem; this makes for fast redisplay
    if (_pattern_loaded)
      {
        CopyFromMemoryDC(_vmem, x, y, x, y, w, h);
      }

    // pass up the hierarchy, just in case
    vCanvasPane::Redraw(x,y,w,h);

  }


//=====================>>> vpatviewCanvasPane::Resize <<<======================
  void vpatviewCanvasPane::Resize(int w, int h)
  {
    VCursor old_cursor;
    int redo = 0;

    // If resizing to something larger than the current _vmem, we need to 
    // resize _vmem to handle it (and only increase size, never decrease it)
    if (w > _vmem_x )
      {
        redo = 1;
        _vmem_x = w;
      }

    if (h > _vmem_y )
      {
        redo = 1;
        _vmem_y = h;
      }

    if (redo == 1)
      {
        redo = 0;                       // just to be safe (this is probably
                                        // over-paranoid)
        delete _vmem;
        _vmem = new vMemoryDC(_vmem_x, _vmem_y);
      }

    old_cursor = GetCursor();
    SetCursor(VC_Wait);

// DrawScaled() and Redraw() handled by UpdateScreen()
    UpdateScreen();

    SetCursor(old_cursor);

    vCanvasPane::Resize(w, h);

  }

//====================>>> vpatviewCanvasPane::SetChart <<<=====================
  void vpatviewCanvasPane::SetChart(vpat_chart *pat)
  {
    if (_pat)
      {
        if (_pat->palette) free(_pat->palette);
        if (_pat->stitches) free(_pat->stitches);
        if (_pat->bstitches) free(_pat->bstitches);
        _pat->palette = NULL;
        _pat->stitches = NULL;
        _pat->bstitches = NULL;
        free(_pat);
      }
    _pat = pat;
  }

//===================>>> vpatviewCanvasPane::DrawScaled <<<====================
  void vpatviewCanvasPane::DrawScaled(vDC *cp)
  {
    float wid, hei, scale_x, scale_y, scale, inc_x, inc_y;
    int ix, iy, bst, w, h, off_x, off_y, size, stitch_index, 
        last_color, bst_scale;
    vPoint vp[4];

// The math is hopefully self-explanatory, but just in case here's a 
// map of the function:
//
//     Calculate size of final drawing (with default margin)
//     Calculate scaling; if scaling > some reasonable number (8), 
//         recalculate margins as if 8 is factor (and set factor to 8)
//     If scaling < 1, draw pixels
//       else draw rectangles/triangles
//     Use scaling to determine backstitch width, and draw backstitches

    if (!_pat) return;

    if (_pattern_loaded)
      {
        cp->SetBackground(_bgcolor);            // this sets the background 
      }

    cp->Clear();                                // to do the Right Thing

    off_x = 5;                                  // default margin of 5
    off_y = 5;                                  // default margin of 5

    w = GetWidth();
    h = GetHeight();

    wid = (float)(w - (off_x*2));               // calculate width to draw
    hei = (float)(h - (off_y*2));               // calculate height to draw

    scale_x = wid/(float)_pat->header.width;         // get scaling factors
    scale_y = hei/(float)_pat->header.height;

    if (scale_x < scale_y)
      {
        scale = scale_x;
        off_y = (int)((float)h - ((float)_pat->header.height * scale_x)) / 2;
      }
    else
      {
        scale = scale_y;
        off_x = (int)((float)w - ((float)_pat->header.width * scale_y)) / 2;
      }

    if (_pat->stitches) 
      {

        _pen.SetStyle(vTransparent);
        _pen.SetWidth(0);
        cp->SetPen(_pen);

        _brush.SetStyle(vSolid);
        cp->SetBrush(_brush);

        inc_x = 0.0;
        inc_y = 0.0;

        if (scale <= 1.0)
          {

            _pen.SetStyle(vSolid);
            _pen.SetWidth(1);
            cp->SetPen(_pen);

            for (iy = 0; iy < _pat->header.height; iy++)
              {
                inc_x = 0.0;
                for (ix = 0; ix < _pat->header.width; ix++)
                  {
                    stitch_index = (iy*_pat->header.width) + ix;
                    if (_pat->stitches[stitch_index].color != -1) 
                                                     // magic value: no stitch
                      {

                        _pen.SetColor(
                                  _colors[_pat->stitches[stitch_index].color] );
                        cp->SetPen(_pen);

                        cp->DrawPoint(((int)inc_x) + off_x, 
                                      ((int)inc_y) + off_y);
                      }

                    inc_x += scale;

                  }

                inc_y += scale;

              }

          }

        else

          {

            _pen.SetStyle(vTransparent);
            _pen.SetWidth(0);
            cp->SetPen(_pen);

            size = (int)(scale + 1.0);

            for (iy = 0; iy < _pat->header.height; iy++)
              {

                inc_x = 0.0;

                for (ix = 0; ix < _pat->header.width; ix++)
                  {
                    stitch_index = (iy*_pat->header.width) + ix;
                    if (_pat->stitches[stitch_index].color != -1)
                                                     // magic value: no stitch
                      {

                        switch (_pat->stitches[stitch_index].type)
                          {
                            case 0:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              cp->DrawRectangle(
                                      ((int)inc_x) + off_x,
                                      ((int)inc_y) + off_y,
                                      size, size);
                              break;

                            case 7:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].second]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x + size;
                              vp[0].y = (int)inc_y + off_y + size;
                              vp[1].x = (int)inc_x + off_x + size;
                              vp[1].y = (int)inc_y + off_y;
                              vp[2].x = (int)inc_x + off_x;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x + size;
                              vp[3].y = (int)inc_y + off_y + size;
                              cp->DrawPolygon(4, vp);

                            case 1:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x;
                              vp[0].y = (int)inc_y + off_y;
                              vp[1].x = (int)inc_x + off_x + size;
                              vp[1].y = (int)inc_y + off_y;
                              vp[2].x = (int)inc_x + off_x;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x;
                              vp[3].y = (int)inc_y + off_y;
                              cp->DrawPolygon(4, vp);
                              break;
  
                           case 8:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].second]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x;
                              vp[0].y = (int)inc_y + off_y;
                              vp[1].x = (int)inc_x + off_x;
                              vp[1].y = (int)inc_y + off_y + size;
                              vp[2].x = (int)inc_x + off_x + size;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x;
                              vp[3].y = (int)inc_y + off_y;
                              cp->DrawPolygon(4, vp);

                           case 2:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x;
                              vp[0].y = (int)inc_y + off_y;
                              vp[1].x = (int)inc_x + off_x + size;
                              vp[1].y = (int)inc_y + off_y;
                              vp[2].x = (int)inc_x + off_x + size;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x;
                              vp[3].y = (int)inc_y + off_y;
                              cp->DrawPolygon(4, vp);
                              break;

                           case 3:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x;
                              vp[0].y = (int)inc_y + off_y;
                              vp[1].x = (int)inc_x + off_x + size;
                              vp[1].y = (int)inc_y + off_y + size;
                              vp[2].x = (int)inc_x + off_x;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x;
                              vp[3].y = (int)inc_y + off_y;
                              cp->DrawPolygon(4, vp);
                              break;

                           case 4:
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              vp[0].x = (int)inc_x + off_x;
                              vp[0].y = (int)inc_y + off_y + size;
                              vp[1].x = (int)inc_x + off_x + size;
                              vp[1].y = (int)inc_y + off_y;
                              vp[2].x = (int)inc_x + off_x + size;
                              vp[2].y = (int)inc_y + off_y + size;
                              vp[3].x = (int)inc_x + off_x;
                              vp[3].y = (int)inc_y + off_y + size;
                              cp->DrawPolygon(4, vp);
                              break;

                           case 6: // HALF STITCH (/)
                              _brush.SetColor(_colors[_pat->
                                              stitches[stitch_index].color]);
                              cp->SetBrush(_brush);

                              cp->DrawLine((int)inc_x + off_x,
                                           (int)inc_y + off_y + size,
                                           (int)inc_x + off_x + size,
                                           (int)inc_y + off_y);
                              break;

                            default:
                              break;
                          }

                      }

                    inc_x += scale;

                  }

                inc_y += scale;

              }

          }

      } 

    if (_pat->bstitches)
      {

        bst = (int)(scale/6.0) + 1;
        bst_scale = (int)(scale/2.0);
                                    // all back stitches are stored at 2
                                    // location, to take care of incremental
                                    // pattern file types

        _pen.SetStyle(vSolid);
        _pen.SetWidth(bst);
        _pen.SetColor(vStdColors[vC_Black]);
        cp->SetPen(_pen);
        last_color=-1;

        for (ix = 0; ix < (_pat->header.num_bstitches); ix++)
          {
            if (last_color != _pat->bstitches[ix].color)
              {
                if (_pat->bstitches[ix].color == -1)
                  {
                    _pen.SetColor(vStdColors[vC_Black]);
                  }
                else
                  {
                    _pen.SetColor(_colors[_pat->bstitches[ix].color]);
                  }
                last_color=_pat->bstitches[ix].color;
              }

            cp->SetPen(_pen);

            if (_pat->bstitches[ix].type == 0)
              {
                cp->DrawLine((int)((float)_pat->bstitches[ix].x1 * scale / 2.0)
                                  + off_x,
                             (int)((float)_pat->bstitches[ix].y1 * scale / 2.0)
                                  + off_y,
                             (int)((float)_pat->bstitches[ix].x2 * scale / 2.0)
                                  + off_x,
                             (int)((float)_pat->bstitches[ix].y2 * scale / 2.0)
                                  + off_y);
              }

            else if (_pat->bstitches[ix].type == 1)

              {
                // Draw Frech Knot
                _brush.SetColor(_colors[_pat->bstitches[ix].color]);
                cp->SetBrush(_brush);
                cp->DrawEllipse((int)((float)(_pat->bstitches[ix].x1 + 1) * 
                                  bst_scale)/2
                                  + off_x - ((int)scale)/2,
                                (int)((float)(_pat->bstitches[ix].y1 + 1) * 
                                  bst_scale)/2
                                  + off_y - ((int)scale)/2,
                                (int)scale - 1, (int)scale - 1);
              }

          }

      }

}

//================>>> vpatviewCanvasPane::UpdateScreen <<<=====================

  void vpatviewCanvasPane::UpdateScreen()
  {

    DrawScaled(_vmem);
    if (_pattern_loaded)
      {
        _vmem->SetBackground(_bgcolor);
      }

    if (_pattern_loaded)
      {
        SetBackground(_bgcolor);
      }
    Clear();
    Redraw(0,0,GetWidth(),GetHeight());

  }

