//=============================================================================
//
//      vpat_fnt.cpp:
//
// This file contains a very, very large function, which draws a symbol 
// on a DC passed to it, based on the ASCII character passed to it.
// 
// Note that this *requires* that the printer scaling patch is installed.
//
// This is included for use with the X version of vpatview, and it is 
// *not* supported under Windows.  
//
//=============================================================================

#include "vpat_fnt.h"

#include <v/vpen.h>
#include <v/vbrush.h>

// Some basic assumptions for this:
//   (1) The canvas pane is currently set to scale 1/1
//   (2) The pen will be set to black herein, but it will need to be
//       *reset* after exit
//   (3) The (x,y) is the *CHART* level square
//   (4) st_type is 0 for full, then 1 2
//                                   3 4
//   (5) scale is the width of the cell, in ordinary pixels

const int fnt_scale_factor = 4;

void vfnt_draw_symbol( vDC *cp, char ch, int x, int y, int scale, int st_type)
{

#ifdef _V_SCALED_PRINTER

  int old_mult, old_div, old_trans_x, old_trans_y;

  cp->GetScale(old_mult, old_div);
  old_trans_x = cp->GetTransX();
  old_trans_y = cp->GetTransY();

  cp->SetTransX(old_trans_x * fnt_scale_factor);
  cp->SetTransY(old_trans_y * fnt_scale_factor);

  cp->SetScale(1, fnt_scale_factor);

// draw the symbol
  vPen fnt_pen;
  vBrush fnt_brush;
  int fnt_scale = scale * fnt_scale_factor;

// coordinates
  int fnt_x, fnt_y, fnt_w, fnt_h;
  vPoint vp[25];                      // adjust this upwards if anything gets
                                      // *really* complicated

// "scratch" variables for the big switch()
  int _margin, _size, _sizem, _quarter, _half;

  fnt_x = x * fnt_scale_factor * scale + 1; // +1 = account for grid
  fnt_y = y * fnt_scale_factor * scale + 1; // +1 = account for grid
  fnt_w = fnt_scale;
  fnt_h = fnt_scale;

//  _margin = (scale / 8) + (fnt_scale_factor*2);
  switch (scale)
    {

      case 1:
      case 2:
      case 3:
      case 4:
        _margin = 2;
        break;

      case 5:
      case 6:
        _margin = 4;
        break;

      default:
        _margin = 8;
        break;

    }

  _size = fnt_scale - _margin*2; //(int)((float)_margin * 1.5);
  _sizem = _size + _margin;
  _half = fnt_scale / 2;
  _quarter = fnt_scale / 4;

  switch ((unsigned char)ch)
    {

       case   0:     // *

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);

        break;

       case   1:      // box with X

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;


       case   2:      // I
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin*2, fnt_y + _margin,
                     fnt_x + _size, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin*2, fnt_y + _sizem,
                     fnt_x + _size, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);

        break;

       case   3:      // |-| (horizontal I)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin*2,
                     fnt_x + _margin, fnt_y + _size);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin*2,
                     fnt_x + _sizem, fnt_y + _size);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);

        break;

       case   4:      // 

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        cp->DrawEllipse(fnt_x + _quarter, fnt_y + _quarter,
                        _half, _half);

        break;

       case   5:      // 

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        cp->DrawEllipse(fnt_x + _quarter, fnt_y + _margin,
                        _half, _half);

        break;

       case   6:      // 

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _quarter,
                        _half, _half);

        break;

       case   7:      // 

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);

        cp->DrawEllipse(fnt_x + _half, fnt_y + _quarter,
                        _half, _half);

        break;

// FIXME: insert 8 - 23

       case  24:      // filled box
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _size, _size);

        break;

       case  25:      // hollow box
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _size, _size);

        break;


       case  26:      // -
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);

        break;


       case  27:      // \ 
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  28:      // horizontal oval
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _half - _margin,
                        _size, _margin*2);

        break;

       case  29:      // vertical oval
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _half - _margin, fnt_y + _margin,
                        _margin*2, _size);

        break;

       case  30:      // +

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);

        break;

       case  31:      // X

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

       case  32:      // solid circle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                          _size, _size);

        break;

       case  33:      // vertical line (e.g. |)

        fnt_pen.SetWidth(2);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);

        break;

       case  34:       // down-left arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

       case  35:       // down-right arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  36:       // up-left arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _size);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  37:       // up-right arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

       case  38:       // circle with + through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  39:       // circle with | through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  40:       // circle with \ through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  41:       // circle with - through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  42:       // circle with / through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  43:       // circle with X through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _size, _size);

        break;

       case  44:       // up-left arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin, 
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  45:       // up-right arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _sizem;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

       case  46:       // down-left arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

       case  47:       // down-right arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _sizem;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _sizem;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  48:       // solid diamond

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _half;
        vp[4].x = fnt_x + _half;
        vp[4].y = fnt_y + _margin;

        cp->DrawPolygon(5, vp);

        break;

       case  49:       // hollow diamond

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _half;
        vp[4].x = fnt_x + _half;
        vp[4].y = fnt_y + _margin;

        cp->DrawPolygon(5, vp);

        break;

       case  50:       // solid triangle pointing left <|

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _sizem;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _sizem;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  51:       // solid triangle pointing right |>

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  52:       // solid triangle pointing up ^

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _half;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case 53:       // solid triangle pointing down V

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  54:       // hollow triangle pointing left <|

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _sizem;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _sizem;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  55:       // hollow triangle pointing right |>

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  56:       // hollow triangle pointing up ^

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _half;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

       case  57:       // hollow triangle pointing down V

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case  58:        // /

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

       case  59:       // /\ with dot

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _margin);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _size - _margin/2,
                        _margin, _margin);

        break;

       case  60:       // \/ with dot

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _margin,
                        _margin, _margin);

        break;

       case  61:       // < with dot

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _half);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

       case  62:       // > with dot

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _size - _margin/2, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

       case  63:       // ->

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _half);

        break;

       case  64:       // ->

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                     fnt_x + _margin, fnt_y + _half);

        break;

      case 'A':               // up-left hollow arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'B':               // up-right hollow arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'C':               // down-left hollow arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'D':               // down-right hollow arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'E':               // up-left solid triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'F':               // up-right solid triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _sizem;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'G':               // down-left solid triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'H':               // down-right solid triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _sizem;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _sizem;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _sizem;

        cp->DrawPolygon(4, vp);

        break;

      case 'I':               // +--
                              // |
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'J':               // --+
                              //   |
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);

        break;

      case 'K':               // |
                              // +--
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'L':               //   |
                              // --+
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'M':               // up-left hollow triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'N':               // up-right hollow triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _sizem;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'O':               // down-left hollow triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _margin;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _margin;

        cp->DrawPolygon(4, vp);

        break;

      case 'P':               // down-right hollow triangle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _sizem;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _margin;
        vp[2].x = fnt_x + _sizem;
        vp[2].y = fnt_y + _sizem;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _sizem;

        cp->DrawPolygon(4, vp);

        break;

      case 'Q':               // +--
                              // | o
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'R':               // --+
                              // o |
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'S':               // | o
                              // +--
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'T':               // o |
                              // --+
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'U':               // '\ (line w/ circle)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                     _margin, _margin);

        break;

      case 'V':               // /' (line w/ circle)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _margin,
                     _margin, _margin);

        break;

      case 'W':               // ./ (line w/ circle)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _size,
                     _margin, _margin);

        break;

      case 'X':               // \. (line w/ circle)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin, 
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _size,
                     _margin, _margin);

        break;

      case 'Y':               // | with circles on both ends

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _margin,
                     _margin, _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _size,
                     _margin, _margin);

        break;

      case 'Z':               // *--*

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half, 
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _half - _margin,
                     _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _half - _margin,
                     _margin, _margin);

        break;

      case  91:               // up arrow (line)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _half, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                     fnt_x + _half, fnt_y + _margin);

        break;

      case  92:               // down arrow (line)

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                     fnt_x + _half, fnt_y + _sizem);

        break;

      case  93:               // hollow left-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _half;
        vp[1].x = fnt_x + _half;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _quarter*3;
        vp[3].x = fnt_x + _sizem;
        vp[3].y = fnt_y + _quarter*3;
        vp[4].x = fnt_x + _sizem;
        vp[4].y = fnt_y + _quarter;
        vp[5].x = fnt_x + _half;
        vp[5].y = fnt_y + _quarter;
        vp[6].x = fnt_x + _half;
        vp[6].y = fnt_y + _margin;
        vp[7].x = fnt_x + _margin;
        vp[7].y = fnt_y + _half;

        cp->DrawPolygon(8, vp);

        break;

      case  94:               // hollow right-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _sizem;
        vp[0].y = fnt_y + _half;
        vp[1].x = fnt_x + _half;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _size;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _size;
        vp[4].x = fnt_x + _margin;
        vp[4].y = fnt_y + _margin*2;
        vp[5].x = fnt_x + _half;
        vp[5].y = fnt_y + _margin*2;
        vp[6].x = fnt_x + _half;
        vp[6].y = fnt_y + _margin;
        vp[7].x = fnt_x + _sizem;
        vp[7].y = fnt_y + _half;

        cp->DrawPolygon(8, vp);

        break;

      case  95:               // hollow up-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _size;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _size;
        vp[3].y = fnt_y + _sizem;
        vp[4].x = fnt_x + _margin*2;
        vp[4].y = fnt_y + _sizem;
        vp[5].x = fnt_x + _margin*2;
        vp[5].y = fnt_y + _half;
        vp[6].x = fnt_x + _margin;
        vp[6].y = fnt_y + _half;
        vp[7].x = fnt_x + _half;
        vp[7].y = fnt_y + _margin;

        cp->DrawPolygon(8, vp);

        break;

      case  96:               // hollow down-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _sizem;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _size;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _size;
        vp[3].y = fnt_y + _margin;
        vp[4].x = fnt_x + _margin*2;
        vp[4].y = fnt_y + _margin;
        vp[5].x = fnt_x + _margin*2;
        vp[5].y = fnt_y + _half;
        vp[6].x = fnt_x + _margin;
        vp[6].y = fnt_y + _half;
        vp[7].x = fnt_x + _half;
        vp[7].y = fnt_y + _sizem;

        cp->DrawPolygon(8, vp);

        break;

      case 'a':               // /\ with dot at top
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _margin);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _half - _margin/2, 
                        fnt_y + _margin,
                        _margin, _margin);

        break;

      case 'b':               // \/ with dot at bottom
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2,
                        fnt_y + _size,
                        _margin, _margin);

        break;

      case 'c':               // < with dot at side
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                     fnt_x + _margin, fnt_y + _half);
        cp->DrawEllipse(fnt_x + _margin,
                        fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'd':               // > with dot at side
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawEllipse(fnt_x + _size,
                        fnt_y + _half - _margin/2,
                        _margin, _margin);

        break;

      case 'e':               // Y with = through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _half,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _margin, fnt_y + _size,
                     fnt_x + _sizem, fnt_y + _size);

        break;

      case 'f':               // upside-down Y with = through it
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _half,
                     fnt_x + _half, fnt_y + _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin*2,
                     fnt_x + _sizem, fnt_y + _margin*2);

        break;

      case 'g':               // >-||
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _size, fnt_y + _margin,
                     fnt_x + _size, fnt_y + _sizem);

        break;

      case 'h':               // ||-<
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);

        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                     fnt_x + _half, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _half,
                     fnt_x + _margin, fnt_y + _half);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin*2, fnt_y + _margin,
                     fnt_x + _margin*2, fnt_y + _sizem);

        break;

      case 'i':               // '.

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _size,
                        _margin, _margin);

        break;

      case 'j':               // .'

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _size, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _margin, fnt_y + _size,
                        _margin, _margin);

        break;

      case 'k':               // :

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _size,
                        _margin, _margin);

        break;

      case 'l':               // ..

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _half,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _half,
                        _margin, _margin);

        break;

      case 'm':               // : with - through it

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _half - _margin/2, fnt_y + _size,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);

        break;

      case 'n':               // .|.

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _half,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _half,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);

        break;

      case 'o':               // '/.

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin, 
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _size,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

      case 'p':               // .\'

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _size,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'q':               // +--
                              // | #
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _half, fnt_y + _half, 
                          _half - _margin, _half - _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);

        break;


      case 'r':               // --+
                              // # |
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _half,
                          _half - _margin, _half - _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _margin);

        break;


      case 's':               // | #
                              // +--
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _half, fnt_y + _margin, 
                          _half - _margin, _half - _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 't':               // # |
                              // --+
        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _half - _margin, _half - _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'u':               // box with | through it

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _sizem, _sizem);
        cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                     fnt_x + _half, fnt_y + _sizem);

        break;

      case 'v':               // box with - through it

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _sizem, _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                     fnt_x + _sizem, fnt_y + _half);

        break;

      case 'w':               // box with / through it

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                          _sizem, _sizem);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

      case 'x':               // box with \ through it

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin, 
                          _sizem, _sizem);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'y':               // '\.

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _size,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                     fnt_x + _sizem, fnt_y + _sizem);

        break;

      case 'z':               // ./'

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _size,
                        _margin, _margin);
        cp->DrawEllipse(fnt_x + _size, fnt_y + _margin,
                        _margin, _margin);
        cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                     fnt_x + _margin, fnt_y + _sizem);

        break;

      case 123:               // solid left-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _margin;
        vp[0].y = fnt_y + _half;
        vp[1].x = fnt_x + _half;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _size;
        vp[3].x = fnt_x + _sizem;
        vp[3].y = fnt_y + _size;
        vp[4].x = fnt_x + _sizem;
        vp[4].y = fnt_y + _margin*2;
        vp[5].x = fnt_x + _half;
        vp[5].y = fnt_y + _margin*2;
        vp[6].x = fnt_x + _half;
        vp[6].y = fnt_y + _margin;
        vp[7].x = fnt_x + _margin;
        vp[7].y = fnt_y + _half;

        cp->DrawPolygon(8, vp);

        break;

      case 124:               // solid right-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _sizem;
        vp[0].y = fnt_y + _half;
        vp[1].x = fnt_x + _half;
        vp[1].y = fnt_y + _sizem;
        vp[2].x = fnt_x + _half;
        vp[2].y = fnt_y + _size;
        vp[3].x = fnt_x + _margin;
        vp[3].y = fnt_y + _size;
        vp[4].x = fnt_x + _margin;
        vp[4].y = fnt_y + _margin*2;
        vp[5].x = fnt_x + _half;
        vp[5].y = fnt_y + _margin*2;
        vp[6].x = fnt_x + _half;
        vp[6].y = fnt_y + _margin;
        vp[7].x = fnt_x + _sizem;
        vp[7].y = fnt_y + _half;

        cp->DrawPolygon(8, vp);

        break;

      case 125:               // solid up-arrow

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);

        vp[0].x = fnt_x + _half;
        vp[0].y = fnt_y + _margin;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _size;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _size;
        vp[3].y = fnt_y + _sizem;
        vp[4].x = fnt_x + _margin*2;
        vp[4].y = fnt_y + _sizem;
        vp[5].x = fnt_x + _margin*2;
        vp[5].y = fnt_y + _half;
        vp[6].x = fnt_x + _margin;
        vp[6].y = fnt_y + _half;
        vp[7].x = fnt_x + _half;
        vp[7].y = fnt_y + _margin;

        cp->DrawPolygon(8, vp);

        break;

      case 126:               // solid down-arrow
      
        fnt_pen.SetWidth(1);      
        cp->SetPen(fnt_pen);      
        fnt_brush.SetStyle(vSolid);
        cp->SetBrush(fnt_brush);      
      
        vp[0].x = fnt_x + _half;      
        vp[0].y = fnt_y + _sizem;
        vp[1].x = fnt_x + _sizem;
        vp[1].y = fnt_y + _half;
        vp[2].x = fnt_x + _size;
        vp[2].y = fnt_y + _half;
        vp[3].x = fnt_x + _size;
        vp[3].y = fnt_y + _margin;
        vp[4].x = fnt_x + _margin*2;
        vp[4].y = fnt_y + _margin;
        vp[5].x = fnt_x + _margin*2;
        vp[5].y = fnt_y + _half;
        vp[6].x = fnt_x + _margin;
        vp[6].y = fnt_y + _half;
        vp[7].x = fnt_x + _half;      
        vp[7].y = fnt_y + _sizem;
      
        cp->DrawPolygon(8, vp);      
      
        break; 

       case 127:      // hollow circle

        fnt_pen.SetWidth(1);
        cp->SetPen(fnt_pen);
        fnt_brush.SetStyle(vTransparent);
        cp->SetBrush(fnt_brush);

        cp->DrawEllipse(fnt_x + _margin, fnt_y + _margin,
                          _size, _size);

        break;

        case 128:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);

          break;

        case 129:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);

          break;

        case 130:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _margin;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          vp[0].x = fnt_x + _sizem;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _sizem;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          break;

        case 131:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _margin;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _sizem;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _sizem;
          cp->DrawPolygon(4, vp);

          break;

        case 132:

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                       _half - _margin, _half - _margin);

          break;

        case 133:

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);

          cp->DrawRectangle(fnt_x + _half, fnt_y + _margin,
                       _half - _margin, _half - _margin);

          break;

        case 134:

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _half,
                       _half - _margin, _half - _margin);

          break;

        case 135:

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          cp->DrawRectangle(fnt_x + _half, fnt_y + _half,
                       _half - _margin, _half - _margin);

          break;

        case 136:     // sideways block T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _half - _margin;
          vp[3].x = fnt_x + _sizem;
          vp[3].y = fnt_y + _half - _margin;
          vp[4].x = fnt_x + _sizem;
          vp[4].y = fnt_y + _half + _margin;
          vp[5].x = fnt_x + _half;
          vp[5].y = fnt_y + _half + _margin;
          vp[6].x = fnt_x + _half;
          vp[6].y = fnt_y + _sizem;
          vp[7].x = fnt_x + _margin;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);

          break;

        case 137:     // sideways block T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _sizem;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _half - _margin;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _half - _margin;
          vp[4].x = fnt_x + _margin;
          vp[4].y = fnt_y + _half + _margin;
          vp[5].x = fnt_x + _half;
          vp[5].y = fnt_y + _half + _margin;
          vp[6].x = fnt_x + _half;
          vp[6].y = fnt_y + _sizem;
          vp[7].x = fnt_x + _sizem;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _sizem;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);
          break;

         case 138:     // upside-down block T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _sizem;
          vp[1].x = fnt_x + _margin;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _half - _margin;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _half - _margin;
          vp[3].y = fnt_y + _margin;
          vp[4].x = fnt_x + _half + _margin;
          vp[4].y = fnt_y + _margin;
          vp[5].x = fnt_x + _half + _margin;
          vp[5].y = fnt_y + _half;
          vp[6].x = fnt_x + _sizem;
          vp[6].y = fnt_y + _half;
          vp[7].x = fnt_x + _sizem;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _sizem;
          cp->DrawPolygon(9, vp);
          break;

         case 139:     // block T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _half + _margin;
          vp[3].y = fnt_y + _half;
          vp[4].x = fnt_x + _half + _margin;
          vp[4].y = fnt_y + _sizem;
          vp[5].x = fnt_x + _half - _margin;
          vp[5].y = fnt_y + _sizem;
          vp[6].x = fnt_x + _half - _margin;
          vp[6].y = fnt_y + _half;
          vp[7].x = fnt_x + _margin;
          vp[7].y = fnt_y + _half;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);
          break;

         case 140:     // sideways hollow T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _half - _margin;
          vp[3].x = fnt_x + _sizem;
          vp[3].y = fnt_y + _half - _margin;
          vp[4].x = fnt_x + _sizem;
          vp[4].y = fnt_y + _half + _margin;
          vp[5].x = fnt_x + _half;
          vp[5].y = fnt_y + _half + _margin;
          vp[6].x = fnt_x + _half;
          vp[6].y = fnt_y + _sizem;
          vp[7].x = fnt_x + _margin;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);
          break;

         case 141:     // sideways hollow T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _sizem;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _half - _margin;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _half - _margin;
          vp[4].x = fnt_x + _margin;
          vp[4].y = fnt_y + _half + _margin;
          vp[5].x = fnt_x + _half;
          vp[5].y = fnt_y + _half + _margin;
          vp[6].x = fnt_x + _half;
          vp[6].y = fnt_y + _sizem;
          vp[7].x = fnt_x + _sizem;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _sizem;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);
          break;

         case 142:     // upside-down hollow T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _sizem;
          vp[1].x = fnt_x + _margin;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _half - _margin;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _half - _margin;
          vp[3].y = fnt_y + _margin;
          vp[4].x = fnt_x + _half + _margin;
          vp[4].y = fnt_y + _margin;
          vp[5].x = fnt_x + _half + _margin;
          vp[5].y = fnt_y + _half;
          vp[6].x = fnt_x + _sizem;
          vp[6].y = fnt_y + _half;
          vp[7].x = fnt_x + _sizem;
          vp[7].y = fnt_y + _sizem;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _sizem;
          cp->DrawPolygon(9, vp);
          break;

         case 143:     // hollow T
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _half + _margin;
          vp[3].y = fnt_y + _half;
          vp[4].x = fnt_x + _half + _margin;
          vp[4].y = fnt_y + _sizem;
          vp[5].x = fnt_x + _half - _margin;
          vp[5].y = fnt_y + _sizem;
          vp[6].x = fnt_x + _half - _margin;
          vp[6].y = fnt_y + _half;
          vp[7].x = fnt_x + _margin;
          vp[7].y = fnt_y + _half;
          vp[8].x = fnt_x + _margin;
          vp[8].y = fnt_y + _margin;
          cp->DrawPolygon(9, vp);
          break;

        case 144:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _sizem;
          vp[4].x = fnt_x + _margin;
          vp[4].y = fnt_y + _margin;
          cp->DrawPolygon(5, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;

        case 145:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _sizem;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _half;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _half;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _sizem;
          vp[3].y = fnt_y + _sizem;
          vp[4].x = fnt_x + _sizem;
          vp[4].y = fnt_y + _margin;
          cp->DrawPolygon(5, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 146:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _half;
          vp[4].x = fnt_x + _margin;
          vp[4].y = fnt_y + _margin;
          cp->DrawPolygon(5, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 147:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _sizem;
          vp[1].x = fnt_x + _margin;
          vp[1].y = fnt_y + _half;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _half;
          vp[3].x = fnt_x + _sizem;
          vp[3].y = fnt_y + _sizem;
          vp[4].x = fnt_x + _margin;
          vp[4].y = fnt_y + _sizem;
          cp->DrawPolygon(5, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 148:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _margin;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 149:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _margin;
          vp[2].x = fnt_x + _sizem;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 150:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _margin;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _sizem;
          vp[2].x = fnt_x + _margin;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _margin;
          cp->DrawPolygon(4, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;


        case 151:     // box, filled in on left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          vp[0].x = fnt_x + _sizem;
          vp[0].y = fnt_y + _margin;
          vp[1].x = fnt_x + _sizem;
          vp[1].y = fnt_y + _sizem;
          vp[2].x = fnt_x + _margin;
          vp[2].y = fnt_y + _sizem;
          vp[3].x = fnt_x + _margin;
          vp[3].y = fnt_y + _sizem;
          cp->DrawPolygon(4, vp);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);

          break;

        case 152:     // E with prongs up
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _half, fnt_y + _sizem);

          break;

        case 153:     // E with prongs down
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _half, fnt_y + _margin);

          break;

        case 154:     // E with prongs left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _half);

          break;

        case 155:     // E 
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _margin, fnt_y + _half);

          break;

        case 156:     // W
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _margin);

          break;

        case 157:     // W with prongs down
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

        case 158:     // W with prongs left
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _sizem);

          break;

        case 159:     // W with prongs right
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

        case 160:     // V

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _sizem);

          break;

        case 161:     // ^  (up V)

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _margin);

          break;

        case 162:     // <

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _half);

          break;

        case 163:     // >

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _half);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _half);

          break;

      case 164:       // ||

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin*2, fnt_y + _margin,
                       fnt_x + _margin*2, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _size, fnt_y + _margin,
                       fnt_x + _size, fnt_y + _sizem);

          break;

      case 165:       // =

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin*2,
                       fnt_x + _sizem, fnt_y + _margin*2);
          cp->DrawLine(fnt_x + _margin, fnt_y + _size,
                       fnt_x + _sizem, fnt_y + _size);

          break;

      case 166:       // \\

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin*2, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _size);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin*2,
                       fnt_x + _size, fnt_y + _sizem);

          break;

      case 167:       // //

          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _size, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _size);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin*2,
                       fnt_x + _margin*2, fnt_y + _sizem);

          break;

      case 168:       // Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _half, fnt_y + _sizem);

          break;

      case 169:       // upside-down Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _half, fnt_y + _margin);

          break;

      case 170:       // left Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _half);

          break;

      case 171:       // right Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _half,
                       fnt_x + _margin, fnt_y + _half);

          break;

      case 172:       // up-left Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);

          break;

      case 173:       // up-right Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);

          break;

      case 174:       // down-left Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);

          break;

      case 175:       // down-right Y
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                       fnt_x + _half, fnt_y + _half);
          cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                       fnt_x + _half, fnt_y + _half);

          break;

      case 176:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                            _size, _size);
          cp->DrawEllipse(fnt_x + _quarter, fnt_y + _quarter,
                            _half, _half);

          break;

      case 177:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                            _size, _size);

          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawEllipse(fnt_x + _quarter, fnt_y + _quarter,
                            _half, _half);

          break;

      case 178:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                            _size, _size);
          cp->DrawRectangle(fnt_x + _quarter, fnt_y + _quarter,
                            _half, _half);

          break;

      case 179:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);
          fnt_brush.SetStyle(vTransparent);
          cp->SetBrush(fnt_brush);

          cp->DrawRectangle(fnt_x + _margin, fnt_y + _margin,
                            _size, _size);

          fnt_brush.SetStyle(vSolid);
          cp->SetBrush(fnt_brush);

          cp->DrawRectangle(fnt_x + _quarter, fnt_y + _quarter,
                            _half, _half);

          break;



// FIXME: insert 180 - 183

      case 184:       // K
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _half,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 185:       // backwards K
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _half,
                       fnt_x + _margin, fnt_y + _sizem);

          break;

      case 186:       // K on its back
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _half, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _margin);

          break;

      case 187:       // K on its front
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _half, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 188:       // Z
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 189:       // backwards Z
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 190:       // backwards N
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 191:       // N
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _margin);

          break;

      case 204:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 205:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _margin, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 206:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _sizem, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;

      case 207:
          fnt_pen.SetWidth(1);
          cp->SetPen(fnt_pen);

          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _sizem, fnt_y + _margin);
          cp->DrawLine(fnt_x + _margin, fnt_y + _margin,
                       fnt_x + _margin, fnt_y + _sizem);
          cp->DrawLine(fnt_x + _margin, fnt_y + _sizem,
                       fnt_x + _sizem, fnt_y + _sizem);

          break;



      default:
#ifdef _MSP_DEBUG_
        fprintf(stderr, "[%x]", (unsigned long)ch & 0x000000FFUL);
#endif
        break;

    }

// restore the few settings we mucked with

  cp->SetScale(old_mult, old_div);
  cp->SetTranslate(old_trans_x, old_trans_y);

#endif

}

#ifdef _SAMPLE_SYMBOLS_
void vfnt_symbol_chart(vDC *cp)
{
    int y, x;

    for (y = 0; y < 16; y++)
      for (x = 0; x < 16; x++)
        {
//        this first option just plain doesn't come out right -- it needs
//        to be recalculated (FIXME)
//        vfnt_draw_symbol(cp, (char)(y*16+x), x+4, y+2,  4, 0);
          vfnt_draw_symbol(cp, (char)(y*16+x), x+2, y+4, 10, 0);
          vfnt_draw_symbol(cp, (char)(y*16+x), x+1, y+8, 24, 0);
        }

}
#endif

