//
// vpat_io.cpp
//


//#define _MSP_DEBUG_

#include "vpat_io.h"

#include "io_cse.h"
#include "io_pcst.h"
#include "io_pmak.h"
#include "io_pt5.h"
#include "io_xsw.h"

#ifdef _USE_VSTITCH_
#include "io_vst.h"
#endif

#ifdef MSDOS
#include <io.h>
#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#define O_BINARY 0

#endif

#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>

// change this if any chart type can have more!
vColor _bgcolor(255,255,255);
vColor _colors[VPAT_NUM_COLORS];

int _pattern_loaded = false;                 // no pattern loaded yet

// This needs to be adjusted to the largest size that a given detection 
// routine needs.  A flexible detection routine would make do with less 
// info, but it is passed anyways.
#define CHUNK_SIZE 264

int vpat_ReadChart(vpat_chart *pat, char *name)
  {
    char *chunk;
    int pat_fd, retval, num_read;

    if (!pat) return VPATIO_BADPAT;
    if (!(chunk = (char *)malloc(CHUNK_SIZE))) return VPATIO_MALLOC;
    if ((pat_fd=open(name, O_RDONLY | O_BINARY))<0) return VPATIO_OPEN;

    num_read = read(pat_fd, chunk, CHUNK_SIZE);
    close(pat_fd);

    if (IsPCStitch(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found PC-Stitch\n");
#endif
        retval = ReadPCStitch(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }

    if (IsPMaker(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found PMaker\n");
#endif
        retval = ReadPMaker(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }

    if (IsXSWin(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found XS-Win\n");
#endif
        retval = ReadXSWin(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }

    if (IsWinStitch(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found WinStitch\n");
#endif
        retval = ReadWinStitch(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }

    if (IsPT5(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found PT5\n");
#endif
        retval = ReadPT5(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }

#ifdef _USE_VSTITCH_
    if (IsVStitch(chunk, num_read))
      {
#ifdef _MSP_DEBUG_
        fprintf(stderr, "Found VStitch\n");
#endif
        retval = ReadVStitch(pat, name);
        if (retval == VPATIO_OK) _pattern_loaded = true;
        free(chunk);
        return retval;
      }
#endif

    free(chunk);
    return VPATIO_UNKNOWN;

  }

char _vpat_error_str[256];

char *vpat_IOErrString(int err)
  {
    switch(err)
      {
        case VPATIO_OK:
          sprintf(_vpat_error_str, "No error");
          break;

        case VPATIO_BADPAT:
          sprintf(_vpat_error_str, "Bad pattern handle");
          break;

        case VPATIO_UNKNOWN:
          sprintf(_vpat_error_str, "Unknown pattern type");
          break;

        case VPATIO_OPEN:
          sprintf(_vpat_error_str, "Unable to open file");
          break;

        case VPATIO_READ:
          sprintf(_vpat_error_str, "Error reading from file");
          break;

        case VPATIO_WRITE:
          sprintf(_vpat_error_str, "Error writing to file");
          break;

        case VPATIO_SHORT:
          sprintf(_vpat_error_str, "Short file");
          break;

        case VPATIO_MALLOC:
          sprintf(_vpat_error_str, "Error allocating memory");
          break;

        case VPATIO_UNSUPPORTED:
          sprintf(_vpat_error_str, "Unsupported file version");
          break;

        default:
          sprintf(_vpat_error_str, "Unknown error");
          break;

      }

    return _vpat_error_str;

  }

