//=============================================================================
//  vpat_pge.cpp
//=============================================================================
//

#include "vpat_pge.h"             // our header file
#include <v/vapp.h>
#include <v/vnotice.h>
#include <v/vicon.h>
#include <v/vfontsel.h>           // vFont Selector dialog (V utility class)

    enum                        // Enumerated type for Itemvals
      {
        plDummyItem = 200,      // Dummy to start stuff at 200
        plFrame,
        plFrame2,
        plFrame3,
        plFont,
        plSymFont,
        plIntFont,
        plPrintChart,
        plShowGrid,
        plColorSheet,
        plScaleBy,
        plStitchType,
        plPCLabel,
        plPCFirst,
        plPCLast,
        plMarginLabel,
        plMargin
      };

    char *stitchList[] =        // stitch type list for combobox
      {
        "Colored Box",
        "Colored X",
        "Symbol",
        0
      };

    char *marginList[] =        // margin options
      {
        "1/4\"",
        "1/2\"",
        "3/4\"",
        "1\"",
        0
      };

    int scaleList[3] =
      {
        4, 24, 2                // scaling range: 4..24, in steps of 2
      };

    static CommandObject PageCmds[] =
      {
        {C_Frame,plFrame,0,"Chart Printing Options",NoList,CA_None,isSens,
                NoFrame,notUsed,notUsed},
        {C_CheckBox,plPrintChart,1,"Print Chart",NoList,CA_None,isSens,
                plFrame,notUsed,notUsed},
        {C_CheckBox,plShowGrid,1,"Show Grid",NoList,CA_None,isSens,plFrame,
                notUsed,plPrintChart},
        {C_ComboBox,plStitchType,0,"Stitch Type",stitchList,CA_None,isSens,
                plFrame,plPrintChart,notUsed},
        {C_Spinner,plScaleBy,10,"Scaling Factor",(void *)scaleList,CA_None,
                isSens,plFrame,plStitchType,notUsed},
        {C_Label,plMarginLabel,0,"Margin:",NoList,CA_None,isSens,plFrame,
                plShowGrid,plStitchType},
        {C_ComboBox,plMargin,1,"Printing Margin",marginList,CA_None,isSens,
                plFrame,plMarginLabel,plScaleBy},
        {C_Frame,plFrame3,0,"Color List Options",NoList,CA_None,isSens,
                notUsed,notUsed,plFrame},
        {C_CheckBox,plColorSheet,1,"Color List",NoList,CA_None,isSens,plFrame3,
                notUsed,notUsed},
        {C_Label,plPCLabel,0,"Print Color List:",NoList,CA_None,isSens,
                plFrame3,notUsed,plColorSheet},
        {C_CheckBox,plPCFirst,1,"First",NoList,CA_None,isSens,
                plFrame3,notUsed,plPCLabel},
        {C_CheckBox,plPCLast,0,"Last",NoList,CA_None,isSens,
                plFrame3,plPCFirst,plPCLabel},
        {C_Frame,plFrame2,0,"Font Options",NoList,CA_None,isSens,
                NoFrame,plFrame3,plFrame},
        {C_Button,plFont,plFont,"Text Font",NoList,CA_None,
                isSens,plFrame2,notUsed,notUsed},
        {C_Button,plSymFont,plSymFont,"Symbol Font",NoList,CA_None,
                isSens,plFrame2,notUsed,plFont},
#ifdef _V_SCALED_PRINTER
        {C_CheckBox,plIntFont,0,"Use internal font",NoList,CA_None,
                isSens,plFrame2,notUsed,plSymFont},
#endif
        {C_Button,M_OK,M_OK,"   OK   ",NoList,CA_DefaultButton,
                isSens,NoFrame,notUsed,plFrame3},
        {C_EndOfList,0,0,0,0,CA_None,0,0,0}
      };


//=================>>> pageModalDialog::pageModalDialog <<<====================
  pageModalDialog::pageModalDialog(vBaseWindow* bw) :
    vModalDialog(bw)
  {

    _scale_by = 10;
    _show_grid = 1;
    _color_list = 1;
    _print_type = 0;
    _print_chart = 1;

    _margin = 36;

    _print_colors_first = 1;
    _print_colors_last = 0;


#ifdef _V_SCALED_PRINTER
    _int_font = 0;
#endif

    _font = new vFont(vfDefaultSystem, 10, vfNormal, vfNormal, 0);
    _symfont = new vFont(vfDecorative, 10, vfNormal, vfNormal, 0);

    AddDialogCmds(PageCmds);         // add the predefined commands

  }

//================>>> pageModalDialog::~pageModalDialog <<<====================
  pageModalDialog::~pageModalDialog()
  {

  }

//===================>>> pageModalDialog::DialogCommand <<<====================
  void pageModalDialog::DialogCommand(ItemVal id,ItemVal retval,CmdType ctype)
  {
    // After the user has selected a command from the dialog,
    // this routine is called with the id and retval.

    int dummy;
    vFontSelect *vfs;

    vfs = new vFontSelect(theApp);

    switch(id)
      {

        case plColorSheet:
          _color_list = retval;

          switch(_color_list)
            {

              case 1:
                SetValue(plPCFirst, isSens, Sensitive);
                SetValue(plPCLast, isSens, Sensitive);
                break;

              case 0:
                SetValue(plPCFirst, notSens, Sensitive);
                SetValue(plPCLast, notSens, Sensitive);
                break;

            }

          break;

        case plPCFirst:
          _print_colors_first = retval;

          switch(_print_colors_first)
            {

              case 1:
                _print_colors_last = 0;
                SetValue(plPCLast, 0, Checked);
                break;

              case 0:
                _print_colors_last = 1;
                SetValue(plPCLast, 1, Checked);
                break;

            }

          break;

        case plPCLast:
          _print_colors_last = retval;

          switch(_print_colors_last)
            {

              case 1:
                _print_colors_first = 0;
                SetValue(plPCFirst, 0, Checked);
                break;

              case 0:
                _print_colors_first = 1;
                SetValue(plPCFirst, 1, Checked);
                break;

            }

          break;

        case plPrintChart:
          _print_chart = retval;

          switch(_print_chart)
            {

              case 1:
                SetValue(plShowGrid,isSens,Sensitive);
                SetValue(plStitchType,isSens,Sensitive);
                SetValue(plScaleBy,isSens,Sensitive);

                break;

              case 0:
                SetValue(plShowGrid,notSens,Sensitive);
                SetValue(plStitchType,notSens,Sensitive);
                SetValue(plScaleBy,notSens,Sensitive);
                break;

            }

          break;

        case plShowGrid:
          _show_grid = retval;
          break;

        case plStitchType:
          _print_type = retval;
          break;

        case plMargin:
          switch (retval)
            {
              case 0: // 1/4"
                _margin = 18;
                break;

              case 1:
                _margin = 36;
                break;

              case 2:
                _margin = 54;
                break;

              case 3:
                _margin = 72;
                break;

              default:
                _margin = 36;
                break;
            }

          break;

        case plScaleBy:
          _scale_by = retval;
          _font->SetFontValues(_font->GetFamily(),
                               _scale_by,
                               _font->GetStyle(),
                               _font->GetWeight(),
                               _font->GetUnderlined() );
          _symfont->SetFontValues(_symfont->GetFamily(),
                               _scale_by,
                               _symfont->GetStyle(),
                               _symfont->GetWeight(),
                               _symfont->GetUnderlined() );
          break;

        case plFont:

          dummy = vfs->FontSelect(*_font, 
                                  "Select font for color list & instructions");

          if (_font->GetPointSize() != _scale_by)
          {
            _scale_by = _font->GetPointSize();
            SetValue(plScaleBy,_scale_by,Value);
            _symfont->SetFontValues(_symfont->GetFamily(),
                                 _scale_by,
                                 _symfont->GetStyle(),
                                 _symfont->GetWeight(),
                                 _symfont->GetUnderlined() );
 
          }

          break;

        case plSymFont:

          dummy = vfs->FontSelect(*_symfont,
                                  "Select font for symbol printing");

          if (_symfont->GetPointSize() != _scale_by)
          {
            _scale_by = _symfont->GetPointSize();
            SetValue(plScaleBy,_scale_by,Value);
            _font->SetFontValues(_font->GetFamily(),
                                 _scale_by,
                                 _font->GetStyle(),
                                 _font->GetWeight(),
                                 _font->GetUnderlined() );

          }

          break;

#ifdef _V_SCALED_PRINTER
        case plIntFont:
          _int_font = retval;
          break;
#endif

        default:
          break;
      }

    // all commands should route through parent type
    vModalDialog::DialogCommand(id,retval,ctype);

  }

